/*
 * $Header: $
 * $Revision: $
 * $Date: $
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "HttpClient", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */

package org.apache.commons.httpclient.methods.multipart;

import java.io.OutputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;

/**
 *
 * Abstract class for one Part of a multipart post object.
 *
 * @author <a href="mailto:mattalbright@yahoo.com">Matthew Albright</a>
 *
 */
public abstract class Part
{
    static String boundary = "----------------314159265358979323846";
    static byte[] boundary_bytes = boundary.getBytes();
    static String CRLF = "\r\n";
    static byte[] CRLF_bytes = CRLF.getBytes();
    static String extra = "--";
    static byte[] extra_bytes = extra.getBytes();
    
    public static String getBoundary()  { return(boundary); }
    
    public static void sendLastBoundary(OutputStream out)
         throws IOException
    {
        out.write(extra_bytes);
        out.write(boundary_bytes);
        out.write(extra_bytes);
        out.write(CRLF_bytes);
    }
    
    public static int lengthOfLastBoundary()
         throws IOException
    {
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        
        sendLastBoundary(out);
        
        return(out.size());
    }
    
    public abstract String getName();
    
    protected void sendStart(OutputStream out) 
         throws IOException
    {
        out.write(extra_bytes);
        out.write(boundary_bytes);
        out.write(CRLF_bytes);
    }
    
    protected int lengthOfStart()
         throws IOException
    {
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        
        sendStart(out);
        
        return(out.size());
    }
    
    protected void sendHeader(OutputStream out) 
         throws IOException
    {
        String content_dispos = 
             "Content-Disposition: form-data; name=\"" + getName() + "\"";
    
        out.write(content_dispos.getBytes());
    }
    
    protected int lengthOfHeader()
         throws IOException
    {
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        
        sendHeader(out);
        
        return(out.size());
    }
    
    
    protected void sendEndOfHeader(OutputStream out) 
         throws IOException
    {
        out.write(CRLF_bytes);
        out.write(CRLF_bytes);
    }
    
    protected int lengthOfEndOfHeader()
         throws IOException
    {
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        
        sendEndOfHeader(out);
        
        return(out.size());
    }
    
    
    protected abstract void sendData(OutputStream out) throws IOException;
    
    protected abstract long lengthOfData() throws IOException;
    
    
    protected void sendEnd(OutputStream out) 
         throws IOException
    {
        out.write(CRLF_bytes);
    }
    
    protected int lengthOfEnd()
         throws IOException
    {
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        
        sendEnd(out);
        
        return(out.size());
    }
    
    /* The following 2 methods don't need to be final, but they DO need
       to be overridden as a pair, and the only way to make sure of that
       is to make sure they AREN'T overridden. */
    final public void send(OutputStream out) throws IOException
    {
        sendStart(out);
        sendHeader(out);
        sendEndOfHeader(out);
        sendData(out);
        sendEnd(out);
    }
    
    final public long length() throws IOException
    {
        return(lengthOfStart() + 
               lengthOfHeader() + 
               lengthOfEndOfHeader() +
               lengthOfData() + 
               lengthOfEnd());
    }
}
